/****************************************************************************
** Meta object code from reading C++ file 'greeter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "QLightDM/greeter.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'greeter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8QLightDM7GreeterE_t {};
} // unnamed namespace

template <> constexpr inline auto QLightDM::Greeter::qt_create_metaobjectdata<qt_meta_tag_ZN8QLightDM7GreeterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QLightDM::Greeter",
        "showMessage",
        "",
        "text",
        "QLightDM::Greeter::MessageType",
        "type",
        "showPrompt",
        "QLightDM::Greeter::PromptType",
        "authenticationComplete",
        "autologinTimerExpired",
        "idle",
        "reset",
        "connectToDaemonSync",
        "connectSync",
        "authenticate",
        "username",
        "authenticateAsGuest",
        "authenticateAutologin",
        "authenticateRemote",
        "session",
        "respond",
        "response",
        "cancelAuthentication",
        "cancelAutologin",
        "setLanguage",
        "language",
        "setResettable",
        "resettable",
        "startSessionSync",
        "ensureSharedDataDirSync",
        "authenticated",
        "authenticationUser",
        "defaultSession",
        "selectUser",
        "selectGuest",
        "hostname",
        "osId",
        "osName",
        "osPrettyName",
        "osVersion",
        "osVersionId",
        "motd",
        "hasGuestAccount",
        "locked",
        "PromptType",
        "PromptTypeQuestion",
        "PromptTypeSecret",
        "MessageType",
        "MessageTypeInfo",
        "MessageTypeError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'showMessage'
        QtMocHelpers::SignalData<void(QString, QLightDM::Greeter::MessageType)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'showPrompt'
        QtMocHelpers::SignalData<void(QString, QLightDM::Greeter::PromptType)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 7, 5 },
        }}),
        // Signal 'authenticationComplete'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'autologinTimerExpired'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'idle'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reset'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'connectToDaemonSync'
        QtMocHelpers::SlotData<bool()>(12, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'connectSync'
        QtMocHelpers::SlotData<bool()>(13, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'authenticate'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'authenticate'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'authenticateAsGuest'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'authenticateAutologin'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'authenticateRemote'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'authenticateRemote'
        QtMocHelpers::SlotData<void(const QString &)>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'authenticateRemote'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'respond'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'cancelAuthentication'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cancelAutologin'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setLanguage'
        QtMocHelpers::SlotData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'setResettable'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Slot 'startSessionSync'
        QtMocHelpers::SlotData<bool(const QString &)>(28, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'startSessionSync'
        QtMocHelpers::SlotData<bool()>(28, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'ensureSharedDataDirSync'
        QtMocHelpers::SlotData<QString(const QString &)>(29, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'authenticated'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'authenticationUser'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'defaultSession'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'selectUser'
        QtMocHelpers::PropertyData<QString>(33, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'selectGuest'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'hostname'
        QtMocHelpers::PropertyData<QString>(35, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'osId'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'osName'
        QtMocHelpers::PropertyData<QString>(37, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'osPrettyName'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'osVersion'
        QtMocHelpers::PropertyData<QString>(39, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'osVersionId'
        QtMocHelpers::PropertyData<QString>(40, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'motd'
        QtMocHelpers::PropertyData<QString>(41, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'hasGuestAccount'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'locked'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'PromptType'
        QtMocHelpers::EnumData<enum PromptType>(44, 44, QMC::EnumFlags{}).add({
            {   45, PromptType::PromptTypeQuestion },
            {   46, PromptType::PromptTypeSecret },
        }),
        // enum 'MessageType'
        QtMocHelpers::EnumData<enum MessageType>(47, 47, QMC::EnumFlags{}).add({
            {   48, MessageType::MessageTypeInfo },
            {   49, MessageType::MessageTypeError },
        }),
    };
    return QtMocHelpers::metaObjectData<Greeter, qt_meta_tag_ZN8QLightDM7GreeterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QLightDM::Greeter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8QLightDM7GreeterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8QLightDM7GreeterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8QLightDM7GreeterE_t>.metaTypes,
    nullptr
} };

void QLightDM::Greeter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Greeter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QLightDM::Greeter::MessageType>>(_a[2]))); break;
        case 1: _t->showPrompt((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QLightDM::Greeter::PromptType>>(_a[2]))); break;
        case 2: _t->authenticationComplete(); break;
        case 3: _t->autologinTimerExpired(); break;
        case 4: _t->idle(); break;
        case 5: _t->reset(); break;
        case 6: { bool _r = _t->connectToDaemonSync();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->connectSync();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->authenticate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->authenticate(); break;
        case 10: _t->authenticateAsGuest(); break;
        case 11: _t->authenticateAutologin(); break;
        case 12: _t->authenticateRemote((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 13: _t->authenticateRemote((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->authenticateRemote(); break;
        case 15: _t->respond((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->cancelAuthentication(); break;
        case 17: _t->cancelAutologin(); break;
        case 18: _t->setLanguage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 19: _t->setResettable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: { bool _r = _t->startSessionSync((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->startSessionSync();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { QString _r = _t->ensureSharedDataDirSync((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Greeter::*)(QString , QLightDM::Greeter::MessageType )>(_a, &Greeter::showMessage, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Greeter::*)(QString , QLightDM::Greeter::PromptType )>(_a, &Greeter::showPrompt, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Greeter::*)()>(_a, &Greeter::authenticationComplete, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Greeter::*)()>(_a, &Greeter::autologinTimerExpired, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Greeter::*)()>(_a, &Greeter::idle, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Greeter::*)()>(_a, &Greeter::reset, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isAuthenticated(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->authenticationUser(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->defaultSessionHint(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->selectUserHint(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->selectGuestHint(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->hostname(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->osId(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->osName(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->osPrettyName(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->osVersion(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->osVersionId(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->motd(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->hasGuestAccountHint(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->lockHint(); break;
        default: break;
        }
    }
}

const QMetaObject *QLightDM::Greeter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QLightDM::Greeter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8QLightDM7GreeterE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QLightDM::Greeter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void QLightDM::Greeter::showMessage(QString _t1, QLightDM::Greeter::MessageType _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void QLightDM::Greeter::showPrompt(QString _t1, QLightDM::Greeter::PromptType _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QLightDM::Greeter::authenticationComplete()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QLightDM::Greeter::autologinTimerExpired()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QLightDM::Greeter::idle()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QLightDM::Greeter::reset()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
