/****************************************************************************
** Meta object code from reading C++ file 'usersmodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "QLightDM/usersmodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'usersmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QLightDM__UsersModel_t {
    QByteArrayData data[12];
    char stringdata0[168];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QLightDM__UsersModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QLightDM__UsersModel_t qt_meta_stringdata_QLightDM__UsersModel = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QLightDM::UsersModel"
QT_MOC_LITERAL(1, 21, 14), // "UserModelRoles"
QT_MOC_LITERAL(2, 36, 8), // "NameRole"
QT_MOC_LITERAL(3, 45, 12), // "RealNameRole"
QT_MOC_LITERAL(4, 58, 12), // "LoggedInRole"
QT_MOC_LITERAL(5, 71, 14), // "BackgroundRole"
QT_MOC_LITERAL(6, 86, 11), // "SessionRole"
QT_MOC_LITERAL(7, 98, 15), // "HasMessagesRole"
QT_MOC_LITERAL(8, 114, 13), // "ImagePathRole"
QT_MOC_LITERAL(9, 128, 18), // "BackgroundPathRole"
QT_MOC_LITERAL(10, 147, 7), // "UidRole"
QT_MOC_LITERAL(11, 155, 12) // "IsLockedRole"

    },
    "QLightDM::UsersModel\0UserModelRoles\0"
    "NameRole\0RealNameRole\0LoggedInRole\0"
    "BackgroundRole\0SessionRole\0HasMessagesRole\0"
    "ImagePathRole\0BackgroundPathRole\0"
    "UidRole\0IsLockedRole"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QLightDM__UsersModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       1,   14, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // enums: name, alias, flags, count, data
       1,    1, 0x0,   10,   19,

 // enum data: key, value
       2, uint(QLightDM::UsersModel::NameRole),
       3, uint(QLightDM::UsersModel::RealNameRole),
       4, uint(QLightDM::UsersModel::LoggedInRole),
       5, uint(QLightDM::UsersModel::BackgroundRole),
       6, uint(QLightDM::UsersModel::SessionRole),
       7, uint(QLightDM::UsersModel::HasMessagesRole),
       8, uint(QLightDM::UsersModel::ImagePathRole),
       9, uint(QLightDM::UsersModel::BackgroundPathRole),
      10, uint(QLightDM::UsersModel::UidRole),
      11, uint(QLightDM::UsersModel::IsLockedRole),

       0        // eod
};

void QLightDM::UsersModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QLightDM::UsersModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_meta_stringdata_QLightDM__UsersModel.data,
    qt_meta_data_QLightDM__UsersModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QLightDM::UsersModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QLightDM::UsersModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QLightDM__UsersModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int QLightDM::UsersModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
